@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2018-2023 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2018-2024 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -392,7 +392,9 @@ as_cache_remove_old_data_from_dir (AsCache *cache, const gchar *cache_dir)
 		fname_full = g_build_filename (cache_dir, g_file_info_get_name (finfo), NULL);
 
 		/* jump one directory deeper */
-		if (g_file_info_get_file_type (finfo) == G_FILE_TYPE_DIRECTORY) {
+		if (g_file_info_get_file_type (finfo) == G_FILE_TYPE_DIRECTORY ||
+		    (g_file_info_get_file_type (finfo) == G_FILE_TYPE_SYMBOLIC_LINK &&
+		     g_file_test (fname_full, G_FILE_TEST_IS_DIR))) {
 			g_autoptr(GFileEnumerator) sd_direnum = NULL;
 			g_autoptr(GFile) sd_cdir = NULL;
 
@@ -1736,7 +1738,7 @@ as_cache_get_components_by_provided_item (AsCache *cache,
 	XbValueBindings *vbindings = xb_query_context_get_bindings (&context);
 
 	xpath_query_tmpl = "components/component/provides/%s[text()=?]/../..";
-	xpath_query_type_tmpl = "components/component/provides/%s[text()=?][@type='%s']/../..";
+	xpath_query_type_tmpl = "components/component/provides/%s[@type='%s'][text()=?]/../..";
 
 	if (kind == AS_PROVIDED_KIND_LIBRARY)
 		kind_node_name = "library";
@@ -1968,8 +1970,8 @@ as_cache_search (AsCache *cache, const gchar *const *terms, gboolean sort, GErro
 		for (guint j = 0; queries[j].xpath != NULL; j++) {
 			g_autoptr(GError) error_query = NULL;
 			g_autoptr(XbQuery) query = xb_query_new (csec->silo,
-								  queries[j].xpath,
-								  &error_query);
+								 queries[j].xpath,
+								 &error_query);
 			if (query != NULL) {
 				AsFTSearchHelper *helper = g_new0 (AsFTSearchHelper, 1);
 				helper->match_value = queries[j].match_value;
